import logging
from twisted.internet import reactor, defer
from smpp.twisted.client import SMPPClientTransceiver, SMPPClientService
from smpp.twisted.config import SMPPClientConfig

class SMPP:
    
    def __init__(self, config=None):
        if config is None:
            config = SMPPClientConfig(host='localhost', port=999, username='uname', password='pwd')

            # Uncomment line below to recv SMS via #322223322 only
            # config = SMPPClientConfig(host='localhost', port=999, username='uname', password='pwd', addressTon=AddrTon.UNKNOWN, addressNpi=AddrNpi.ISDN, addressRange='^322223322$')
        self.config = config
        
    @defer.inlineCallbacks
    def run(self):
        try:
            #Bind
            smpp = yield SMPPClientTransceiver(self.config, self.handleMsg).connectAndBind()
            #Wait for disconnect
            yield smpp.getDisconnectedDeferred()
        except Exception, e:
            print "ERROR: %s" % str(e)
        finally:
            reactor.stop()
    
    def handleMsg(self, smpp, pdu):
        """
        NOTE: you can return a Deferred here
        """
        print "Received pdu %s" % pdu
    
if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    SMPP().run()
    reactor.run()



import unittest
import logging
from twisted.internet import defer, task
from twisted.trial.unittest import TestCase
from twisted.cred.portal import IRealm
from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse
from twisted.cred.portal import Portal
from twisted.test import proto_helpers
from zope.interface import implementer
import sys

sys.path.append(".")
sys.path.append("..")
sys.path.append("../..")
from smpp.twisted.config import SMPPServerConfig, SMPPClientConfig
from smpp.twisted.server import SMPPServerFactory, SMPPBindManager
from smpp.twisted.protocol import SMPPSessionStates, DataHandlerResponse
from smpp.twisted.client import SMPPClientTransceiver

from smpp.pdu import pdu_types, operations, pdu_encoding

import mock

logging.basicConfig(level = logging.DEBUG)

def _makeMockServerConnection(key, bind_type):
    mk_server_cnxn = mock.Mock('mock svr cnxn')
    mk_server_cnxn.system_id = key
    mk_server_cnxn.bind_type = bind_type
    return mk_server_cnxn

class SMPPServerFactoryTests(TestCase):

    def setUp(self):
        self.config = mock.Mock('mock smpp config')
        self.config.systems = {'lala': {'max_bindings': 2}}

    def tearDown(self):
        pass

    def test_canOpenNewConnection_transceiver(self):
        server = SMPPServerFactory(self.config, None)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should be able to bind trx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should be able to bind tx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should be able to bind rx as none bound yet')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_transceiver)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx bound')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_transceiver)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertFalse(can_bind, 'Should not be able to bind as two trx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertFalse(can_bind, 'Should not be able to bind as two trx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertFalse(can_bind, 'Should not be able to bind as two trx already bound')

    def test_canOpenNewConnection_transmitter(self):
        server = SMPPServerFactory(self.config, None)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should be able to bind trx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should be able to bind tx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should be able to bind rx as none bound yet')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_transmitter)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one tx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should still be able to bind as only one tx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should still be able to bind only bindings are tx')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_transmitter)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertFalse(can_bind, 'Should not be able to bind as two tx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertFalse(can_bind, 'Should not be able to bind as two tx already bound')
        # NOTE this one different
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should still be able to bind only bindings are tx')

    def test_canOpenNewConnection_receiver(self):
        server = SMPPServerFactory(self.config, None)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should be able to bind trx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should be able to bind tx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should be able to bind rx as none bound yet')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_receiver)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one rx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should still be able to bind only bindings are rx')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one rx bound')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_receiver)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertFalse(can_bind, 'Should not be able to bind as two rx already bound')
        # NOTE this one different
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should still be able to bind only bindings are rx')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertFalse(can_bind, 'Should not be able to bind as two rx already bound')

    def test_canOpenNewConnection_multitypes(self):
        server = SMPPServerFactory(self.config, None)

        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should be able to bind trx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should be able to bind tx as none bound yet')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should be able to bind rx as none bound yet')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_transceiver)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx bound')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_transmitter)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertFalse(can_bind, 'Should not be able to bind as one trx and one tx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertFalse(can_bind, 'Should not be able to bind as one trx and one tx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertTrue(can_bind, 'Should still be able to bind as only one trx and one tx bound')

        mk_server_cnxn = _makeMockServerConnection('lala', pdu_types.CommandId.bind_receiver)
        server.addBoundConnection(mk_server_cnxn)
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transceiver)
        self.assertFalse(can_bind, 'Should not be able to bind as one trx, one tx, and one rx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_transmitter)
        self.assertFalse(can_bind, 'Should not be able to bind as one trx, one tx, and one rx already bound')
        can_bind = server.canOpenNewConnection('lala', pdu_types.CommandId.bind_receiver)
        self.assertFalse(can_bind, 'Should not be able to bind as one trx, one tx, and one rx already bound')


class SMPPBindManagerTests(TestCase):

    def test_getNextBindingForDelivery(self):
        bm = SMPPBindManager('blah')

        # add an initial rx
        mk_server_cnxn_rx1 = _makeMockServerConnection('blah', pdu_types.CommandId.bind_receiver)
        bm.addBinding(mk_server_cnxn_rx1)

        # Get the only rx
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx1, deliverer)

        # Get the only rx again
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx1, deliverer)

        # Add a new rx
        mk_server_cnxn_rx2 = _makeMockServerConnection('blah', pdu_types.CommandId.bind_receiver)
        bm.addBinding(mk_server_cnxn_rx2)

        # Get the new rx
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx2, deliverer)

        # Expect the original rx again
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx1, deliverer)

        # Add a new tx - shouldn't affect deliverer selection at all
        mk_server_cnxn_tx1 = _makeMockServerConnection('blah', pdu_types.CommandId.bind_transmitter)
        bm.addBinding(mk_server_cnxn_tx1)

        # Expect the 2nd rx again
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx2, deliverer)

        # Add a new trx
        mk_server_cnxn_trx1 = _makeMockServerConnection('blah', pdu_types.CommandId.bind_transceiver)
        bm.addBinding(mk_server_cnxn_trx1)

        # Expect the new trx
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_trx1, deliverer)

        # Remove the 1st rx
        bm.removeBinding(mk_server_cnxn_rx1)

        # Expect the 2nd rx again
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx2, deliverer)

        # Expect the new trx
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_trx1, deliverer)

        # Expect the 2nd rx again
        deliverer = bm.getNextBindingForDelivery()
        self.assertEqual(mk_server_cnxn_rx2, deliverer)

class SMPPServerBaseTest(TestCase):
    def _serviceHandler(self, system_id, smpp, pdu):
        self.service_calls.append((system_id, smpp, pdu))
        return pdu_types.CommandStatus.ESME_ROK

    @implementer(IRealm)
    class SmppRealm:

        def requestAvatar(self, avatarId, mind, *interfaces):
            return ('SMPP', avatarId, lambda: None)

    def _bind(self):
        self.proto.bind_type = pdu_types.CommandId.bind_transceiver
        self.proto.sessionState = SMPPSessionStates.BOUND_TRX
        self.proto.system_id = 'userA'
        self.factory.addBoundConnection(self.proto)

class SMPPServerTestCase(SMPPServerBaseTest):

    def setUp(self):
        self.service_calls = []
        self.encoder = pdu_encoding.PDUEncoder()
        self.smpp_config = SMPPServerConfig(msgHandler=self._serviceHandler,
                                            systems={'userA': {"max_bindings": 2}}
                                            )
        portal = Portal(self.SmppRealm())
        credential_checker = InMemoryUsernamePasswordDatabaseDontUse()
        credential_checker.addUser('userA', 'valid')
        portal.registerChecker(credential_checker)
        self.factory = SMPPServerFactory(self.smpp_config, auth_portal=portal)
        self.proto = self.factory.buildProtocol(('127.0.0.1', 0))
        self.tr = proto_helpers.StringTransport()
        self.proto.makeConnection(self.tr)

    def tearDown(self):
        self.proto.connectionLost('test end')

    def testTRXBindRequest(self):
        pdu = operations.BindTransceiver(
            system_id = 'userA',
            password = 'valid',
            seqNum = 1
        )
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.BindTransceiverResp(system_id='userA', seqNum=1)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        connection = self.factory.getBoundConnections('userA')
        self.assertEqual(connection.system_id, 'userA')
        self.assertEqual(connection._binds[pdu_types.CommandId.bind_transceiver][0], self.proto)

    def testTRXBindRequestInvalidSysId(self):
        pdu = operations.BindTransceiver(
            system_id = 'userB',
            password = 'valid',
            seqNum = 1
        )
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.BindTransceiverResp(system_id='userB', seqNum=1, status=pdu_types.CommandStatus.ESME_RINVSYSID)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        connection = self.factory.getBoundConnections('userA')
        self.assertEqual(connection, None)
        connection = self.factory.getBoundConnections('userB')
        self.assertEqual(connection, None)

    def testTRXBindRequestInvalidPassword(self):
        pdu = operations.BindTransceiver(
            system_id = 'userA',
            password = 'invalid',
            seqNum = 1
        )
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.BindTransceiverResp(system_id='userA', seqNum=1, status=pdu_types.CommandStatus.ESME_RINVPASWD)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        connection = self.factory.getBoundConnections('userA')
        self.assertEqual(connection, None)

    def testTransmitterBindRequest(self):
        system_id = 'userA'
        pdu = operations.BindTransmitter(
            system_id = system_id,
            password = 'valid',
            seqNum = 1
        )
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.BindTransmitterResp(system_id='userA', seqNum=1)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        self.tr.clear()
        connection = self.factory.getBoundConnections(system_id)
        self.assertEqual(connection.system_id, system_id)
        self.assertEqual(connection._binds[pdu_types.CommandId.bind_transmitter][0], self.proto)
        bind_manager = self.factory.getBoundConnections(system_id)
        delivery_binding = bind_manager.getNextBindingForDelivery()
        self.assertTrue(delivery_binding is None)

    def testReceiverBindRequest(self):
        system_id = 'userA'
        pdu = operations.BindReceiver(
            system_id = system_id,
            password = 'valid',
            seqNum = 1
        )
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.BindReceiverResp(system_id='userA', seqNum=1)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        self.tr.clear()
        connection = self.factory.getBoundConnections(system_id)
        self.assertEqual(connection.system_id, system_id)
        self.assertEqual(connection._binds[pdu_types.CommandId.bind_receiver][0], self.proto)
        bind_manager = self.factory.getBoundConnections(system_id)
        delivery_binding = bind_manager.getNextBindingForDelivery()
        self.assertTrue(delivery_binding is not None)
        # TODO Identify what should be returned here
        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=6)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.SubmitSMResp(seqNum=6)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))


    def testUnboundSubmitRequest(self):
        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=1)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.SubmitSMResp(status=pdu_types.CommandStatus.ESME_RINVBNDSTS, seqNum=1)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))

    def testUnboundSubmitRequest2(self):
        pdu = operations.EnquireLink(seqNum = 576)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.EnquireLinkResp(status=pdu_types.CommandStatus.ESME_RINVBNDSTS, seqNum=576)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))

    def testTRXUnbindRequest(self):
        self._bind()
        pdu = operations.Unbind(seqNum = 346)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.UnbindResp(seqNum=346)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        connection = self.factory.getBoundConnections('userA')
        self.assertEqual(connection.system_id, 'userA')
        # Still in list of binds as the connection has not been closed yet. is removed after test tearDown
        self.assertEqual(connection._binds[pdu_types.CommandId.bind_transceiver][0].sessionState, SMPPSessionStates.UNBOUND)

    def testTRXSubmitSM(self):
        self._bind()
        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=6)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.SubmitSMResp(seqNum=6)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        system_id, smpp, pdu_notified = self.service_calls.pop()
        self.assertEqual(system_id, self.proto.system_id)
        self.assertEqual(pdu.params['short_message'], pdu_notified.params['short_message'])
        self.assertEqual(pdu.params['source_addr'], pdu_notified.params['source_addr'])
        self.assertEqual(pdu.params['destination_addr'], pdu_notified.params['destination_addr'])

    def testTRXDataSM(self):
        self._bind()
        pdu = operations.DataSM(source_addr='t1', destination_addr='1208230', short_message='tests', seqNum=6)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.DataSMResp(seqNum=6)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        system_id, smpp, pdu_notified = self.service_calls.pop()
        self.assertEqual(system_id, self.proto.system_id)

    def testTRXQuerySM(self):
        def _serviceHandler(system_id, smpp, pdu):
            self.service_calls.append((system_id, smpp, pdu))
            return DataHandlerResponse(status=pdu_types.CommandStatus.ESME_ROK,
                                       message_id='tests',
                                       final_date=None,
                                       message_state=pdu_types.MessageState.ACCEPTED,
                                       error_code=0)
        self.proto.dataRequestHandler = lambda *args, **kwargs: _serviceHandler(self.proto.system_id, *args, **kwargs)
        self._bind()
        pdu = operations.QuerySM(message_id='tests', source_addr='t1', seqNum=23)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.QuerySMResp(message_id='tests', error_code=0, final_date=None, message_state=pdu_types.MessageState.ACCEPTED ,seqNum=23)
        # Does not work as application using library must reply correctly.
        print(self.tr.value())
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        system_id, smpp, pdu_notified = self.service_calls.pop()
        self.assertEqual(system_id, self.proto.system_id)

    def testRecievePduWhileUnbindPending(self):
        self._bind()
        self.proto.unbind()
        expected_pdu = operations.Unbind(seqNum=1)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        self.tr.clear()
        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=6)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.SubmitSMResp(seqNum=6)
        self.assertEqual(self.tr.value(), b'')
        pdu = operations.UnbindResp(seqNum=1)
        self.proto.dataReceived(self.encoder.encode(pdu))

    def testTRXClientUnbindRequestAfterSubmit(self):
        d = defer.Deferred()
        def _serviceHandler(system_id, smpp, pdu):
            logging.debug("%s, %s, %s", system_id, smpp, pdu)
            return d
        self.proto.dataRequestHandler = lambda *args, **kwargs: _serviceHandler(self.proto.system_id, *args, **kwargs)
        self._bind()

        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=1)
        self.proto.dataReceived(self.encoder.encode(pdu))
        pdu = operations.Unbind(seqNum = 52)
        self.proto.dataReceived(self.encoder.encode(pdu))

        #All PDU requests should fail now.
        #Once we fire this we should get our Submit Resp and the unbind Resp
        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='goodbye', seqNum=5)
        self.proto.dataReceived(self.encoder.encode(pdu))

        unbind_resp_pdu = operations.UnbindResp(seqNum=52)
        submit_fail_pdu = operations.SubmitSMResp(status=pdu_types.CommandStatus.ESME_RINVBNDSTS, seqNum=5)
        # We should have a reply here as our service handler should not be called
        self.assertEqual(self.tr.value(), self.encoder.encode(submit_fail_pdu))
        self.tr.clear()

        d.callback(pdu_types.CommandStatus.ESME_ROK)
        #Then we should get our initial message response and the unbind response
        expected_pdu = operations.SubmitSMResp(seqNum=1)
        self.assertEqual(self.tr.value(), b'%s%s' % (self.encoder.encode(expected_pdu), self.encoder.encode(unbind_resp_pdu)))

    def testTRXServerUnbindRequestAfterSubmit(self):
        deferreds = []
        def _serviceHandler(system_id, smpp, pdu):
            d = defer.Deferred()
            deferreds.append(d)
            logging.debug("%s, %s, %s", system_id, smpp, pdu)
            return d
        self.proto.dataRequestHandler = lambda *args, **kwargs: _serviceHandler(self.proto.system_id, *args, **kwargs)
        self._bind()

        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=1)
        self.proto.dataReceived(self.encoder.encode(pdu))
        unbind_d = self.proto.unbind()
        print(self.tr.value())

        pdu2 = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO2', seqNum=2)
        self.proto.dataReceived(self.encoder.encode(pdu))

        self.assertEqual(1, len(deferreds))
        self.assertEqual(self.tr.value(), b'')
        self.tr.clear()
        deferreds[-1].callback(pdu_types.CommandStatus.ESME_ROK)
        deferreds = deferreds[:-1]
        submit_resp_pdu = operations.SubmitSMResp(seqNum=1)

        unbind_pdu = operations.Unbind(seqNum=1)
        # We should have a reply here as our service handler should not be called
        self.assertEqual(self.tr.value(), b'%s%s' % (self.encoder.encode(submit_resp_pdu), self.encoder.encode(unbind_pdu)))
        self.tr.clear()
        pdu = operations.UnbindResp(seqNum=1)
        self.proto.dataReceived(self.encoder.encode(pdu))

class SMPPServerTimeoutTestCase(SMPPServerBaseTest):

    def setUp(self):
        self.service_calls = []
        self.clock = task.Clock()
        self.encoder = pdu_encoding.PDUEncoder()
        self.smpp_config = SMPPServerConfig(msgHandler=self._serviceHandler,
                                            systems={'userA': {"max_bindings": 2}},
                                            enquireLinkTimerSecs=0.1,
                                            responseTimerSecs=0.1
                                            )
        portal = Portal(self.SmppRealm())
        credential_checker = InMemoryUsernamePasswordDatabaseDontUse()
        credential_checker.addUser('userA', 'valid')
        portal.registerChecker(credential_checker)
        self.factory = SMPPServerFactory(self.smpp_config, auth_portal=portal)
        self.proto = self.factory.buildProtocol(('127.0.0.1', 0))
        self.proto.callLater = self.clock.callLater
        self.tr = proto_helpers.StringTransport()
        self.proto.makeConnection(self.tr)

    def testEnquireTimeout(self):
        self._bind()
        pdu = operations.SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO', seqNum=6)
        self.proto.dataReceived(self.encoder.encode(pdu))
        expected_pdu = operations.SubmitSMResp(seqNum=6)
        self.assertEqual(self.tr.value(), self.encoder.encode(expected_pdu))
        self.service_calls.pop()
        self.tr.clear()
        self.clock.advance(0.1)
        self.clock.advance(0.1)
        self.assertEqual(self.proto.sessionState, SMPPSessionStates.UNBIND_PENDING)



from smpp.twisted.protocol import SMPPServerProtocol
from smpp.pdu import pdu_types

from zope.interface import Interface

from twisted.internet.protocol import ServerFactory
# Jasmin update: direct import of UsernamePassword instead of cred
from twisted.cred.credentials import UsernamePassword
from twisted.cred import error
from twisted.internet import defer

import logging
import collections


LOG_CATEGORY="smpp.twisted.server"

#pylint: disable=inherit-non-class
class IAuthenticatedSMPP(Interface):
    pass

# Jasmin update: using UsernamePassword instead of cred.credentials.UsernamePassword
class UsernameAndPasswordAndIP(UsernamePassword):
    def __init__(self, username, password, client_ip_address):
        self.username = username
        self.password = password
        self.client_ip_address = client_ip_address

class SMPPServerFactory(ServerFactory):

    protocol = SMPPServerProtocol

    def __init__(self, config, auth_portal):
        self.config = config
        self.log = logging.getLogger(LOG_CATEGORY)
        # A dict of protocol instances for each of the current connections,
        # indexed by system_id
        self.bound_connections = {}
        self._auth_portal = auth_portal

    def getConfig(self):
        return self.config

    def getBoundConnectionCount(self, system_id):
        if system_id in self.bound_connections:
            return self.bound_connections[system_id].getMaxTransmitReceiveBindCount()
        else:
            return 0

    def getBoundConnectionCountsStr(self, system_id):
        if system_id in self.bound_connections:
            bind_counts = self.bound_connections[system_id].getBindingCountByType()
            bound_connections_count = []
            for key, value in bind_counts.items():
                bound_connections_count.append("%s: %d" % (key, value))
            bound_connections_str = ', '.join(bound_connections_count)
            return bound_connections_str
        else:
            return '0'

    def addBoundConnection(self, connection):
        """
        Add a protocol instance to the list of current connections.
        @param connection: An instance of SMPPServerProtocol
        """
        system_id = connection.system_id
        self.log.debug('Adding SMPP binding for %s' % system_id)
        if not system_id in self.bound_connections:
            self.bound_connections[system_id] = SMPPBindManager(system_id)
        self.bound_connections[system_id].addBinding(connection)
        bind_type = connection.bind_type
        self.log.info("Added %s bind for '%s'. Active binds: %s. Max binds: %s" % (bind_type, system_id, self.getBoundConnectionCountsStr(system_id), self.config.systems[system_id]['max_bindings']))

    def removeConnection(self, connection):
        """
        Remove a protocol instance (SMPP binding) from the list of current connections.
        @param connection: An instance of SMPPServerProtocol
        """
        if connection.system_id is None:
            self.log.debug("SMPP connection attempt failed without binding.")
        else:
            system_id = connection.system_id
            bind_type = connection.bind_type
            self.bound_connections[system_id].removeBinding(connection)
            self.log.info("Dropped %s bind for '%s'. Active binds: %s. Max binds: %s" % (bind_type, system_id, self.getBoundConnectionCountsStr(system_id), self.config.systems[system_id]['max_bindings']))
            # If this is the last binding for this service then remove the BindManager
            if self.bound_connections[system_id].getBindingCount() == 0:
                self.bound_connections.pop(system_id)

    def getBoundConnections(self, system_id):
        return self.bound_connections.get(system_id)

    def login(self, system_id, password, client_ip_address):
        if self._auth_portal is not None:
            return self._auth_portal.login(
                UsernameAndPasswordAndIP(system_id, password, client_ip_address),
                None,
                IAuthenticatedSMPP
            )
        raise error.UnauthorizedLogin()

    def canOpenNewConnection(self, system_id, bind_type):
        """
        Checks if the gateway with the specified system_id can open a new
        connection, as it hasn't exceeded its maximum number of bindings.
        @param bind_type: One of smpp.pdu.pdu_types.CommandId
        """
        existing_bindings_for_id = self.getBoundConnections(system_id)
        if existing_bindings_for_id:
            connections_count = existing_bindings_for_id.getBindingCountForType(bind_type)
            return connections_count < self.config.systems[system_id]['max_bindings']
        else:
            # No existing bindings for this system_id
            return self.config.systems[system_id]['max_bindings'] > 0

    def unbindGateway(self, system_id):
        """ Unbinds and disconnects all the bindings for the given system_id.  """
        bind_mgr = self.getBoundConnections(system_id)
        if bind_mgr:
            unbinds_list = []
            for bind in bind_mgr:
                unbinds_list.append(bind.getDisconnectedDeferred())
                bind.unbindAndDisconnect()
            d = defer.DeferredList(unbinds_list)
        else:
            d = defer.succeed(None)

        return d

    def unbindAndRemoveGateway(self, system_id):
        '''
        Removes a running gateway from the config so they will be unable to rebind.
        Any attempt to bind while unbinding will receive a ESME_RBINDFAIL error.
        '''
        self.config.systems[system_id]['max_bindings'] = 0
        d = self.unbindGateway(system_id)
        d.addCallback(self.removeGatewayFromConfig, system_id)
        return d

    def removeGatewayFromConfig(self, deferred_res, system_id):
        self.config.systems.pop(system_id)
        return deferred_res

class SMPPBindManager:

    def __init__(self, system_id):
        self.system_id = system_id
        self._binds = {pdu_types.CommandId.bind_transceiver: [],
                       pdu_types.CommandId.bind_transmitter: [],
                       pdu_types.CommandId.bind_receiver: []}
        # A queue of the most recent bindings used for delivering messages
        self._delivery_binding_history = collections.deque()

    def addBinding(self, connection):
        """ @param connection: An instance of SMPPServerProtocol """

        bind_type = connection.bind_type
        self._binds[bind_type].append(connection)

    def removeBinding(self, connection):
        """ @param connection: An instance of SMPPServerProtocol """
        bind_type = connection.bind_type
        self._binds[bind_type].remove(connection)

    def getMaxTransmitReceiveBindCount(self):
        return len(self._binds[pdu_types.CommandId.bind_transceiver]) + \
               max(len(self._binds[pdu_types.CommandId.bind_transmitter]),
                   len(self._binds[pdu_types.CommandId.bind_receiver]))

    def getBindingCount(self):
        return sum(len(v) for v in self._binds.values())

    def getBindingCountByType(self):
        ret = {}
        for key, value in self._binds.items():
            ret[key] = len(value)
        return ret

    def __len__(self):
        return self.getBindingCount()

    def __iter__(self):
        vals = []
        [vals.extend(type) for type in self._binds.values()]
        return vals.__iter__()

    def getBindingCountForType(self, bind_type):
        """
        Sum transceiver binds plus receiver or transmitter depending on this type
        @param bind_type: One of smpp.pdu.pdu_types.CommandId
        """
        if bind_type == pdu_types.CommandId.bind_transceiver:
            # Sum of current transceiver binds plus greater of current transmitter or receiver binds
            connections_count = self.getMaxTransmitReceiveBindCount()
        else:
            # Sum of transceiver binds plus existing binds of this type
            connections_count = sum([len(self._binds[bt]) for bt in (pdu_types.CommandId.bind_transceiver, bind_type)])
        return connections_count

    def getNextBindingForDelivery(self):
        """
        Messages inbound (MO) that are to be forwarded to
        the client systems can be sent via transceiver and
        receiver bindings. Call this method to determine which
        binding to send down next so that traffic travels equally
        down the different binds.
        @return smpp protocol or None
        """
        binding = None
        # If we now have more trx/rx bindings than have been used
        # then iterate through our trx/rx binds until we find one
        # that hasn't yet been used
        if len(self._delivery_binding_history) < self.getBindingCountForType(pdu_types.CommandId.bind_receiver):
            for binding in self._binds[pdu_types.CommandId.bind_receiver] + self._binds[pdu_types.CommandId.bind_transceiver]:
                if not binding in self._delivery_binding_history:
                    break
            else:
                binding = None

        # Otherwise send on the last trx/rx binding delivered on, as
        # long as it is still bound
        while binding is None and self._delivery_binding_history:
            # get last binding used
            _binding = self._delivery_binding_history.popleft()
            # check it is still bound
            if _binding in self._binds[_binding.bind_type]:
                # If so then use it
                binding = _binding

        if binding is not None:
            self._delivery_binding_history.append(binding)
        return binding



"""
Copyright 2009-2010 Mozes, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expressed or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

from io import BytesIO
import struct
import logging
import binascii
from enum import Enum

from collections import namedtuple
from smpp.pdu.operations import (
    GenericNack,
    getPDUClass,
    EnquireLink,
    BindTransmitter,
    BindTransceiver,
    BindReceiver,
    Unbind
)
from smpp.pdu.pdu_encoding import PDUEncoder
from smpp.pdu.pdu_types import PDURequest, PDUResponse, PDUDataRequest, CommandStatus
from smpp.pdu.error import (
    SMPPClientConnectionCorruptedError,
    PDUCorruptError,
    PDUParseError,
    SMPPClientSessionStateError,
    SessionStateError,
    SMPPProtocolError,
    SMPPClientError,
    SMPPError,
    SMPPGenericNackTransactionError,
    SMPPTransactionError,
    SMPPRequestTimoutError,
    SMPPSessionInitTimoutError
)
from smpp.pdu.constants import command_status_name_map

from twisted.internet.protocol import Protocol
from twisted.internet import defer, reactor
from twisted.internet.defer import inlineCallbacks
from twisted.cred import error

LOG_CATEGORY = "smpp.twisted.protocol"

SMPPSessionStates = Enum('SMPPSessionStates', 'NONE, OPEN, BIND_TX_PENDING, BOUND_TX, BIND_RX_PENDING, BOUND_RX, BIND_TRX_PENDING, BOUND_TRX, UNBIND_PENDING, UNBIND_RECEIVED, UNBOUND')

SMPPOutboundTxn = namedtuple('SMPPOutboundTxn', 'request, timer, ackDeferred')
SMPPOutboundTxnResult = namedtuple('SMPPOutboundTxnResult', 'smpp, request, response')


def _safelylogOutPdu(content):
    try:
        return binascii.b2a_hex(content)
    except UnicodeEncodeError:
        return "Couldn't log out the pdu content due to non-ascii characters."


class DataHandlerResponse:

    def __init__(self, status, **params):
        self.status = status
        self.params = params


class SMPPProtocolBase(Protocol):
    """Short Message Peer to Peer Protocol v3.4 implementing ESME (client)"""
    version = 0x34

    def __init__(self):
        self.recvBuffer = b""
        self.connectionCorrupted = False
        self.pduReadTimer = None
        self.enquireLinkTimer = None
        self.inactivityTimer = None
        self.dataRequestHandler = None
        self.lastSeqNum = 0
        self.inTxns = {}
        self.outTxns = {}
        self.sessionState = SMPPSessionStates.NONE
        self.encoder = PDUEncoder()
        self.disconnectedDeferred = defer.Deferred()
        # Overriden in tests
        #pylint: disable=no-member
        self.callLater = reactor.callLater
        self.port = None
        self.log = logging.getLogger(LOG_CATEGORY)

    def config(self):
        #pylint: disable=no-member
        return self.factory.getConfig()

    def connectionMade(self):
        """When TCP connection is made
        """
        Protocol.connectionMade(self)
        self.port = self.transport.getHost().port
        # Start the inactivity timer the connection is dropped if we receive no data
        self.activateInactivityTimer()
        self.sessionState = SMPPSessionStates.OPEN
        self.log.warning("SMPP connection established from %s to port %s", self.transport.getPeer().host, self.port)

    def connectionLost(self, reason):
        Protocol.connectionLost(self, reason)
        self.log.warning("SMPP %s disconnected from port %s: %s", self.transport.getPeer().host, self.port, reason)

        self.sessionState = SMPPSessionStates.NONE

        self.cancelEnquireLinkTimer()
        self.cancelInactivityTimer()

        self.disconnectedDeferred.callback(None)

    def dataReceived(self, data):
        """ Looks for a full PDU (protocol data unit) and passes it from
        rawMessageReceived.
        """
        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Received data [%s]" % _safelylogOutPdu(data))

        self.recvBuffer = self.recvBuffer + data

        while True:
            if self.connectionCorrupted:
                return
            msg = self.readMessage()
            if msg is None:
                break
            self.endPDURead()
            self.rawMessageReceived(msg)

        if len(self.recvBuffer) > 0:
            self.incompletePDURead()

    def incompletePDURead(self):
        if self.pduReadTimer and self.pduReadTimer.active():
            return
        self.pduReadTimer = self.callLater(self.config().pduReadTimerSecs, self.onPDUReadTimeout)

    def endPDURead(self):
        if self.pduReadTimer and self.pduReadTimer.active():
            self.pduReadTimer.cancel()

    def readMessage(self):
        pduLen = self.getMessageLength()
        if pduLen is None:
            return None
        return self.getMessage(pduLen)

    def getMessageLength(self):
        if len(self.recvBuffer) < 4:
            return None
        return struct.unpack('!L', self.recvBuffer[:4])[0]

    def getMessage(self, pduLen):
        if len(self.recvBuffer) < pduLen:
            return None

        message = self.recvBuffer[:pduLen]
        self.recvBuffer = self.recvBuffer[pduLen:]
        return message

    def corruptDataRecvd(self, status=CommandStatus.ESME_RINVCMDLEN):
        self.sendPDU(GenericNack(status=status))
        self.onCorruptConnection()

    def onCorruptConnection(self):
        """ Once the connection is corrupt, the PDU boundaries are lost and it's impossible to
            continue processing messages.
                - Set a flag to indicate corrupt connection
                    - no more parse attempts should be made for inbound data
                    - no more outbound requests should be attempted (they should errback immediately)
                - Cancel outstanding outbound requests (which have not yet been ack'ed)
                    (removed from the list and errback called)
                - Shutdown
        """
        self.log.critical("Connection is corrupt!!! Shutting down...")
        self.connectionCorrupted = True
        self.cancelOutboundTransactions(SMPPClientConnectionCorruptedError())
        self.shutdown()

    def getHeader(self, message):
        try:
            return self.encoder.decodeHeader(BytesIO(message[:self.encoder.HEADER_LEN]))
        except:
            return {}

    def onPDUReadTimeout(self):
        self.log.critical('PDU read timed out. Buffer is now considered corrupt')
        self.corruptDataRecvd()

    def rawMessageReceived(self, message):
        """Called once a PDU (protocol data unit) boundary is identified.

        Creates an SMPP PDU class from the data and calls PDUReceived dispatcher
        """
        pdu = None
        try:
            pdu = self.encoder.decode(BytesIO(message))
        except PDUCorruptError as e:
            self.log.exception(e)
            self.log.critical("Received corrupt PDU %s" % _safelylogOutPdu(message))
            self.corruptDataRecvd(status=e.status)
        except PDUParseError as e:
            self.log.exception(e)
            self.log.critical("Received unparsable PDU %s" % _safelylogOutPdu(message))
            header = self.getHeader(message)
            seqNum = header.get('sequence_number', None)
            commandId = header.get('command_id', None)
            self.sendPDU(getPDUClass(commandId).requireAck(seqNum=seqNum, status=e.status))
        else:
            self.PDUReceived(pdu)

    def PDUReceived(self, pdu):
        """Dispatches incoming PDUs
        """
        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Received PDU: %s" % pdu)

        encoded = self.encoder.encode(pdu)

        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Receiving data [%s]" % _safelylogOutPdu(encoded))

        # Signal SMPP operation
        self.onSMPPOperation()

        if isinstance(pdu, PDURequest):
            self.PDURequestReceived(pdu)
        elif isinstance(pdu, PDUResponse):
            self.PDUResponseReceived(pdu)
        else:
            getattr(self, "onPDU_%s" % pdu.id.name)(pdu)

    def PDURequestReceived(self, reqPDU):
        """Handle incoming request PDUs
        """
        if isinstance(reqPDU, PDUDataRequest):
            self.PDUDataRequestReceived(reqPDU)
            return

        getattr(self, "onPDURequest_%s" % reqPDU.id.name)(reqPDU)

    def onPDURequest_enquire_link(self, reqPDU):
        self.sendResponse(reqPDU)

    def onPDURequest_unbind(self, reqPDU):
        # Allow no more outbound data requests
        # Accept no more inbound requests
        self.sessionState = SMPPSessionStates.UNBIND_RECEIVED
        self.cancelEnquireLinkTimer()
        # Cancel outbound requests
        self.cancelOutboundTransactions(SMPPClientSessionStateError('Unbind received'))
        # Wait for inbound requests to finish then ack and disconnect
        self.finishInboundTxns().addCallback(lambda r: (self.sendResponse(reqPDU) or True) and self.disconnect())

    def sendResponse(self, reqPDU, status=CommandStatus.ESME_ROK, **params):
        self.sendPDU(reqPDU.requireAck(reqPDU.seqNum, status, **params))

    def PDUDataRequestReceived(self, reqPDU):
        if self.sessionState == SMPPSessionStates.UNBIND_PENDING:
            self.log.info("Unbind is pending...Ignoring data request PDU %s" % reqPDU)
            return

        if not self.isBound():
            errMsg = 'Received data request when not bound %s' % reqPDU
            self.cancelOutboundTransactions(SessionStateError(errMsg, CommandStatus.ESME_RINVBNDSTS))
            return self.fatalErrorOnRequest(reqPDU, errMsg, CommandStatus.ESME_RINVBNDSTS)

        if self.dataRequestHandler is None:
            return self.fatalErrorOnRequest(reqPDU, 'Missing dataRequestHandler', CommandStatus.ESME_RX_T_APPN)

        self.doPDURequest(reqPDU, self.dataRequestHandler)

    def fatalErrorOnRequest(self, reqPDU, errMsg, status):
        self.log.critical(errMsg)
        self.sendResponse(reqPDU, status)
        self.shutdown()

    def doPDURequest(self, reqPDU, handler):
        self.startInboundTransaction(reqPDU)

        handlerCall = defer.maybeDeferred(handler, self, reqPDU)
        handlerCall.addCallback(self.PDURequestSucceeded, reqPDU)
        handlerCall.addErrback(self.PDURequestFailed, reqPDU)
        handlerCall.addBoth(self.PDURequestFinished, reqPDU)

    def PDURequestSucceeded(self, dataHdlrResp, reqPDU):
        if reqPDU.requireAck:
            status = CommandStatus.ESME_ROK
            params = {}
            if dataHdlrResp:
                if dataHdlrResp in list(CommandStatus):
                    status = dataHdlrResp
                elif isinstance(dataHdlrResp, DataHandlerResponse):
                    status = dataHdlrResp.status
                    params = dataHdlrResp.params
                else:
                    self.log.critical("Invalid response type returned from data handler %s" % type(dataHdlrResp))
                    status = CommandStatus.ESME_RX_T_APPN
                    self.shutdown()

            self.sendResponse(reqPDU, status, **params)

    def PDURequestFailed(self, error, reqPDU):
        if error.check(SMPPProtocolError):
            # Get the original error
            try:
                error.raiseException()
            except SMPPProtocolError as validation_error:
                self.log.info("Application raised error '%s', forwarding to client. Inbound PDU was [%s], hex[%s]" % (
                validation_error, reqPDU, _safelylogOutPdu(self.encoder.encode(reqPDU))))
                # Jasmin update: validation_error have attribute named commandStatusName
                # return_cmd_status = validation_error.commandStatusName
                return_cmd_status = validation_error.status
                shutdown = False
        else:
            self.log.critical('Exception raised handling inbound PDU [%s] hex[%s]: %s' % (
            reqPDU, _safelylogOutPdu(self.encoder.encode(reqPDU)), error))
            return_cmd_status = CommandStatus.ESME_RX_T_APPN
            shutdown = True

        if reqPDU.requireAck:
            self.sendResponse(reqPDU, return_cmd_status)

        if shutdown:
            self.shutdown()

    def PDURequestFinished(self, result, reqPDU):
        self.endInboundTransaction(reqPDU)
        return result

    def finishTxns(self):
        return defer.DeferredList([self.finishInboundTxns(), self.finishOutboundTxns()])

    def finishInboundTxns(self):
        return defer.DeferredList(self.inTxns.values())

    def finishOutboundTxns(self):
        return defer.DeferredList([txn.ackDeferred for txn in list(self.outTxns.values())])

    def PDUResponseReceived(self, pdu):
        """Handle incoming response PDUs
        """
        if isinstance(pdu, GenericNack):
            self.log.critical("Recevied generic_nack %s" % pdu)
            if pdu.seqNum is None:
                self.onCorruptConnection()
                return

        if pdu.seqNum not in self.outTxns:
            self.log.critical('Response PDU received with unknown outbound transaction sequence number %s' % pdu)
            return

        self.endOutboundTransaction(pdu)

    def sendPDU(self, pdu):
        """Send a SMPP PDU
        """
        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Sending PDU: %s" % pdu)
        encoded = self.encoder.encode(pdu)

        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Sending data [%s]" % _safelylogOutPdu(encoded))

        self.transport.write(encoded)
        self.onSMPPOperation()

    def sendBindRequest(self, pdu):
        return self.sendRequest(pdu, self.config().sessionInitTimerSecs)

    def sendRequest(self, pdu, timeout):
        return defer.maybeDeferred(self.doSendRequest, pdu, timeout)

    def doSendRequest(self, pdu, timeout):
        if self.connectionCorrupted:
            raise SMPPClientConnectionCorruptedError()
        if not isinstance(pdu, PDURequest) or pdu.requireAck is None:
            raise SMPPClientError("Invalid PDU to send: %s" % pdu)

        pdu.seqNum = self.claimSeqNum()
        self.sendPDU(pdu)
        return self.startOutboundTransaction(pdu, timeout)

    def onSMPPOperation(self):
        """Called whenever an SMPP PDU is sent or received
        """
        if self.isBound():
            self.activateEnquireLinkTimer()

        self.activateInactivityTimer()

    def activateEnquireLinkTimer(self):
        if self.enquireLinkTimer and self.enquireLinkTimer.active():
            self.enquireLinkTimer.reset(self.config().enquireLinkTimerSecs)
        elif self.config().enquireLinkTimerSecs:
            self.enquireLinkTimer = self.callLater(self.config().enquireLinkTimerSecs, self.enquireLinkTimerExpired)

    def activateInactivityTimer(self):
        if self.inactivityTimer and self.inactivityTimer.active():
            self.inactivityTimer.reset(self.config().inactivityTimerSecs)
        elif self.config().inactivityTimerSecs:
            self.inactivityTimer = self.callLater(self.config().inactivityTimerSecs, self.inactivityTimerExpired)

    def cancelEnquireLinkTimer(self):
        if self.enquireLinkTimer and self.enquireLinkTimer.active():
            self.enquireLinkTimer.cancel()
            self.enquireLinkTimer = None

    def cancelInactivityTimer(self):
        if self.inactivityTimer and self.inactivityTimer.active():
            self.inactivityTimer.cancel()
            self.inactivityTimer = None

    def enquireLinkTimerExpired(self):
        txn = self.sendRequest(EnquireLink(), self.config().responseTimerSecs)
        txn.addErrback(self.enquireLinkErr)

    def enquireLinkErr(self, failure):
        # Unbinding already anyway. No need to raise another error
        failure.trap(SMPPError)

    def inactivityTimerExpired(self):
        self.log.critical("Inactivity timer expired...shutting down")
        self.shutdown()

    def isBound(self):
        return self.sessionState in (
        SMPPSessionStates.BOUND_TX, SMPPSessionStates.BOUND_RX, SMPPSessionStates.BOUND_TRX)

    def shutdown(self):
        """ Unbind if appropriate and disconnect """

        if self.isBound() and not self.connectionCorrupted:
            self.log.warning("Shutdown requested...unbinding")
            self.unbind().addBoth(lambda result: self.disconnect())
        elif self.sessionState not in (SMPPSessionStates.UNBIND_RECEIVED, SMPPSessionStates.UNBIND_PENDING):
            self.log.warning("Shutdown requested...disconnecting")
            self.disconnect()
        else:
            self.log.debug("Shutdown already in progress")

    def startInboundTransaction(self, reqPDU):
        if reqPDU.seqNum in self.inTxns:
            raise SMPPProtocolError('Duplicate message id [%s] received.  Already in progess.' % reqPDU.seqNum,
                                    CommandStatus.ESME_RUNKNOWNERR)
        txnDeferred = defer.Deferred()
        self.inTxns[reqPDU.seqNum] = txnDeferred
        self.log.debug("Inbound transaction started with message id %s" % reqPDU.seqNum)
        return txnDeferred

    def endInboundTransaction(self, reqPDU):
        if not reqPDU.seqNum in self.inTxns:
            raise ValueError('Unknown inbound sequence number in transaction for request PDU %s' % reqPDU)

        self.log.debug("Inbound transaction finished with message id %s" % reqPDU.seqNum)
        self.inTxns[reqPDU.seqNum].callback(reqPDU)
        del self.inTxns[reqPDU.seqNum]

    def startOutboundTransaction(self, reqPDU, timeout):
        if reqPDU.seqNum in self.outTxns:
            raise ValueError('Seq number [%s] is already in progess.' % reqPDU.seqNum)

        # Create callback deferred
        ackDeferred = defer.Deferred()
        # Create response timer
        timer = self.callLater(timeout, self.onResponseTimeout, reqPDU, timeout)
        # Save transaction
        self.outTxns[reqPDU.seqNum] = SMPPOutboundTxn(reqPDU, timer, ackDeferred)
        self.log.debug("Outbound transaction started with message id %s" % reqPDU.seqNum)
        return ackDeferred

    def closeOutboundTransaction(self, seqNum):
        self.log.debug("Outbound transaction finished with message id %s" % seqNum)

        if seqNum in self.outTxns:
            txn = self.outTxns[seqNum]
            # Remove txn
            del self.outTxns[seqNum]
            # Cancel response timer
            if txn.timer.active():
                txn.timer.cancel()
            return txn
        else:
            self.log.critical('Cannot close outbound transaction: trx id [%s] not found !', seqNum)
            return None

    def endOutboundTransaction(self, respPDU):
        txn = self.closeOutboundTransaction(respPDU.seqNum)

        if txn is not None:
            if respPDU.status == CommandStatus.ESME_ROK:
                if not isinstance(respPDU, txn.request.requireAck):
                    txn.ackDeferred.errback(SMPPProtocolError(respPDU,
                        "Invalid PDU response type [%s] returned for request type [%s]" % (
                        type(respPDU), type(txn.request))))
                    return
                # Do callback
                txn.ackDeferred.callback(SMPPOutboundTxnResult(self, txn.request, respPDU))
                return

            if isinstance(respPDU, GenericNack):
                txn.ackDeferred.errback(SMPPGenericNackTransactionError(respPDU, txn.request))
                return

            errCode = respPDU.status
            txn.ackDeferred.errback(SMPPTransactionError(respPDU, txn.request))

    def endOutboundTransactionErr(self, reqPDU, error):
        self.log.error(error)
        txn = self.closeOutboundTransaction(reqPDU.seqNum)

        if txn is not None:
            # Do errback
            txn.ackDeferred.errback(error)

    def cancelOutboundTransactions(self, error):
        for txn in list(self.outTxns.values()):
            self.endOutboundTransactionErr(txn.request, error)

    def onResponseTimeout(self, reqPDU, timeout):
        errMsg = 'Request timed out after %s secs: %s' % (timeout, reqPDU)
        self.endOutboundTransactionErr(reqPDU, SMPPRequestTimoutError(errMsg))
        self.shutdown()

    def claimSeqNum(self):
        self.lastSeqNum += 1
        return self.lastSeqNum

    def unbindSucceeded(self, result):
        self.sessionState = SMPPSessionStates.UNBOUND
        self.log.warning("Unbind succeeded")
        return result

    def unbindFailed(self, reason):
        self.log.error("Unbind failed [%s]. Disconnecting..." % reason)
        self.disconnect()
        if reason.check(SMPPRequestTimoutError):
            raise SMPPSessionInitTimoutError(str(reason))
        return reason

    def unbindAfterInProgressTxnsFinished(self, result, unbindDeferred):
        self.log.warning('Issuing unbind request')
        self.sendBindRequest(Unbind()).addCallbacks(self.unbindSucceeded, self.unbindFailed).chainDeferred(
            unbindDeferred)

    ############################################################################
    # Public command functions
    ############################################################################
    def unbind(self):
        """Unbind from SMSC

        Result is a Deferred object
        """
        if not self.isBound():
            return defer.fail(
                SMPPClientSessionStateError('unbind called with illegal session state: %s' % self.sessionState))

        self.cancelEnquireLinkTimer()

        self.log.info('Waiting for in-progress transactions to finish...')

        # Signal that
        #   - no new data requests should be sent
        #   - no new incoming data requests should be accepted
        self.sessionState = SMPPSessionStates.UNBIND_PENDING

        unbindDeferred = defer.Deferred()
        # Wait for any in-progress txns to finish
        self.finishTxns().addCallback(self.unbindAfterInProgressTxnsFinished, unbindDeferred)
        # Result is the deferred for the unbind txn
        return unbindDeferred

    def unbindAndDisconnect(self):
        """Unbind from SMSC and disconnect

        Result is a Deferred object
        """
        return self.unbind().addBoth(lambda result: self.disconnect())

    def disconnect(self):
        """Disconnect from SMSC
        """
        if self.isBound():
            self.log.warning("Disconnecting while bound to SMSC...")
        else:
            self.log.warning("Disconnecting...")
        self.sessionState = SMPPSessionStates.UNBOUND
        self.transport.loseConnection()

    def getDisconnectedDeferred(self):
        """Get a Deferred so you can be notified on disconnect
        """
        return self.disconnectedDeferred

    def sendDataRequest(self, pdu):
        """Send a SMPP Request Message

        Argument is an SMPP PDUDataRequest (protocol data unit).
        Result is a Deferred object
        """
        if not isinstance(pdu, PDUDataRequest):
            return defer.fail(SMPPClientError("Invalid PDU passed to sendDataRequest(): %s" % pdu))
        if not self.isBound():
            return defer.fail(SMPPClientSessionStateError('Not bound'))
        return self.sendRequest(pdu, self.config().responseTimerSecs)


class SMPPClientProtocol(SMPPProtocolBase):

    def __init__(self):
        self.log = logging.getLogger(LOG_CATEGORY)
        SMPPProtocolBase.__init__(self)

        self.alertNotificationHandler = None

    def PDUReceived(self, pdu):
        """Dispatches incoming PDUs
        """
        self.log.info("SMPP Client received PDU [command: %s, sequence_number: %s, command_status: %s]" % (
        pdu.id, pdu.seqNum, pdu.status))
        SMPPProtocolBase.PDUReceived(self, pdu)

    def bind(self, pdu, pendingState, boundState):
        if self.sessionState != SMPPSessionStates.OPEN:
            return defer.fail(
                SMPPClientSessionStateError('bind called with illegal session state: %s' % self.sessionState))

        bindDeferred = self.sendBindRequest(pdu)
        bindDeferred.addCallback(self.bindSucceeded, boundState)
        bindDeferred.addErrback(self.bindFailed)
        self.sessionState = pendingState
        return bindDeferred

    def doBindAsReceiver(self):
        self.log.warning('Requesting bind as receiver')
        pdu = BindReceiver(
            system_id=self.config().username,
            password=self.config().password,
            system_type=self.config().systemType,
            address_range=self.config().addressRange,
            addr_ton=self.config().addressTon,
            addr_npi=self.config().addressNpi,
            interface_version=self.version
        )
        return self.bind(pdu, SMPPSessionStates.BIND_RX_PENDING, SMPPSessionStates.BOUND_RX)

    def bindSucceeded(self, result, nextState):
        self.sessionState = nextState
        self.log.warning("Bind succeeded...now in state %s" % str(self.sessionState))
        self.activateEnquireLinkTimer()
        return result

    def bindFailed(self, reason):
        self.log.error("Bind failed [%s]. Disconnecting..." % reason)
        self.disconnect()
        if reason.check(SMPPRequestTimoutError):
            raise SMPPSessionInitTimoutError(str(reason))
        return reason

    def onPDU_outbind(self, pdu):
        if self.sessionState != SMPPSessionStates.OPEN:
            self.log.critical('Received outbind command in invalid state %s' % str(self.sessionState))
            self.shutdown()
            return

        self.log.warning("Received outbind command")
        self.doBindAsReceiver()

    def onPDU_alert_notification(self, pdu):
        if self.sessionState == SMPPSessionStates.UNBIND_PENDING:
            self.log.info("Unbind is pending...Ignoring alert notification PDU %s" % pdu)
            return

        if not self.isBound():
            errMsg = 'Received alert notification when not bound %s' % pdu
            self.cancelOutboundTransactions(SessionStateError(errMsg, CommandStatus.ESME_RINVBNDSTS))
            self.log.critical(errMsg)
            self.shutdown()
            return

        if self.alertNotificationHandler:
            try:
                self.alertNotificationHandler(self, pdu)
            except Exception as e:
                self.log.critical('Alert handler threw exception: %s' % str(e))
                self.log.exception(e)
                self.shutdown()

    ############################################################################
    # Public command functions
    ############################################################################
    def bindAsTransmitter(self):
        """Bind to SMSC as transmitter

        Result is a Deferred object
        """
        self.log.warning('Requesting bind as transmitter')
        pdu = BindTransmitter(
            system_id=self.config().username,
            password=self.config().password,
            system_type=self.config().systemType,
            address_range=self.config().addressRange,
            addr_ton=self.config().addressTon,
            addr_npi=self.config().addressNpi,
            interface_version=self.version
        )
        return self.bind(pdu, SMPPSessionStates.BIND_TX_PENDING, SMPPSessionStates.BOUND_TX)

    def bindAsReceiver(self, dataRequestHandler):
        """Bind to SMSC as receiver

        Result is a Deferred object
        """
        self.setDataRequestHandler(dataRequestHandler)
        return self.doBindAsReceiver()

    def bindAsTransceiver(self, dataRequestHandler):
        """Bind to SMSC as transceiver

        Result is a Deferred object
        """
        self.setDataRequestHandler(dataRequestHandler)
        self.log.warning('Requesting bind as transceiver')
        pdu = BindTransceiver(
            system_id=self.config().username,
            password=self.config().password,
            system_type=self.config().systemType,
            address_range=self.config().addressRange,
            addr_ton=self.config().addressTon,
            addr_npi=self.config().addressNpi,
            interface_version=self.version
        )
        return self.bind(pdu, SMPPSessionStates.BIND_TRX_PENDING, SMPPSessionStates.BOUND_TRX)

    def setDataRequestHandler(self, handler):
        """Set handler to use for receiving data requests
        """
        self.dataRequestHandler = handler

    def setAlertNotificationHandler(self, handler):
        """Set handler to use for receiving data requests
        """
        self.alertNotificationHandler = handler


class SMPPServerProtocol(SMPPProtocolBase):

    def __init__(self):
        SMPPProtocolBase.__init__(self)

        # Jasmin update: dataRequestHandler is set from factory instead of config()
        # Divert received messages to the handler defined in the config
        self.dataRequestHandler = lambda *args, **kwargs: self.config().msgHandler(self.system_id, *args, **kwargs)

        self.system_id = None
        self.log = logging.getLogger(LOG_CATEGORY)

    def onResponseTimeout(self, reqPDU, timeout):
        errMsg = 'Request timed out for system id %s after %s secs: %s' % (self.system_id, timeout, reqPDU)
        self.endOutboundTransactionErr(reqPDU, SMPPRequestTimoutError(errMsg))
        self.shutdown()

    def connectionLost(self, reason):
        # Remove this connection from those stored in the factory
        #pylint: disable=no-member
        self.factory.removeConnection(self)
        SMPPProtocolBase.connectionLost(self, reason)

    def PDUReceived(self, pdu):
        """Dispatches incoming PDUs
        """
        self.log.debug(
            "SMPP Server received PDU to system '%s' [command: %s, sequence_number: %s, command_status: %s]" % (
            self.system_id, pdu.id, pdu.seqNum, pdu.status))
        SMPPProtocolBase.PDUReceived(self, pdu)

    def onPDURequest_enquire_link(self, reqPDU):
        if self.isBound():
            self.sendResponse(reqPDU)
        else:
            self.sendResponse(reqPDU, status=CommandStatus.ESME_RINVBNDSTS)

    def onPDURequest_bind_receiver(self, reqPDU):
        self.doBindRequest(reqPDU, SMPPSessionStates.BOUND_RX)

    def onPDURequest_bind_transmitter(self, reqPDU):
        self.doBindRequest(reqPDU, SMPPSessionStates.BOUND_TX)

    def onPDURequest_bind_transceiver(self, reqPDU):
        self.doBindRequest(reqPDU, SMPPSessionStates.BOUND_TRX)

    @inlineCallbacks
    def doBindRequest(self, reqPDU, sessionState):
        # Check the authentication
        # Decode from byte strings to strings
        system_id, password = reqPDU.params['system_id'].decode(), reqPDU.params['password'].decode()

        # Authenticate system_id and password
        try:
            #pylint: disable=no-member
            iface, auth_avatar, logout = yield self.factory.login(system_id, password, self.transport.getPeer().host)
        except error.UnauthorizedLogin:
            #pylint: disable=no-member
            if system_id not in list(self.factory.config.systems):
                self.log.warning('SMPP Bind request failed for system_id: "%s", System ID not configured' % system_id)
                self.sendErrorResponse(reqPDU, CommandStatus.ESME_RINVSYSID, system_id)
            else:
                self.log.warning('SMPP Bind request failed for system_id: "%s", failed to authenticate' % system_id)
                self.sendErrorResponse(reqPDU, CommandStatus.ESME_RINVPASWD, system_id)
            return

        # Check we're not already bound, and are open to being bound
        if self.sessionState != SMPPSessionStates.OPEN:
            self.log.warning('Duplicate SMPP bind request received from: %s' % system_id)
            self.sendErrorResponse(reqPDU, CommandStatus.ESME_RALYBND, system_id)
            return

        # Check that system_id hasn't exceeded number of allowed binds
        bind_type = reqPDU.commandId
        #pylint: disable=no-member
        if not self.factory.canOpenNewConnection(system_id, bind_type):
            self.log.warning('SMPP System %s has exceeded maximum number of %s bindings' % (system_id, bind_type))
            self.sendErrorResponse(reqPDU, CommandStatus.ESME_RBINDFAIL, system_id)
            return

        # If we get to here, bind successfully
        self.system_id = system_id
        self.sessionState = sessionState
        self.bind_type = bind_type

        #pylint: disable=no-member
        self.factory.addBoundConnection(self)
        bound_cnxns = self.factory.getBoundConnections(system_id)
        self.log.info('Bind request succeeded for %s. %d active binds' % (
        system_id, bound_cnxns.getBindingCount() if bound_cnxns else 0))
        self.sendResponse(reqPDU, system_id=system_id)

    def sendErrorResponse(self, reqPDU, status, system_id):
        """ Send an error response to reqPDU, with the specified command status."""
        err_pdu = reqPDU.requireAck(seqNum=reqPDU.seqNum, status=status, system_id=system_id)
        self.sendPDU(err_pdu)



"""
Copyright 2009-2010 Mozes, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expressed or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import logging
from OpenSSL import SSL
from twisted.internet.protocol import ClientFactory
from twisted.internet import defer, reactor, ssl
from twisted.application.service import Service
from smpp.twisted.protocol import SMPPClientProtocol, DataHandlerResponse

LOG_CATEGORY="smpp.twisted.client"

class SMPPClientFactory(ClientFactory):

    protocol = SMPPClientProtocol

    def __init__(self, config):
        self.config = config
        self.buildProtocolDeferred = defer.Deferred()
        self.log = logging.getLogger(LOG_CATEGORY)

    def getConfig(self):
        return self.config

    def buildProtocol(self, addr):
        p = ClientFactory.buildProtocol(self, addr)
        #This is a sneaky way of passing the protocol instance back to the caller
        #pylint: disable=no-member
        reactor.callLater(0, self.buildProtocolDeferred.callback, p)
        return p

    def clientConnectionFailed(self, connector, reason):
        """Connection failed
        """
        self.log.error("Connection failed. Reason: %s" % str(reason))
        self.buildProtocolDeferred.errback(reason)

class CtxFactory(ssl.ClientContextFactory):

    def __init__(self, config):
        self.smppConfig = config

    def getContext(self):
        self.method = SSL.SSLv23_METHOD
        ctx = ssl.ClientContextFactory.getContext(self)
        if self.smppConfig.SSLCertificateFile:
            ctx.use_certificate_file(self.smppConfig.SSLCertificateFile)
        return ctx

class SMPPClientBase:
    msgHandler = None

    def __init__(self, config):
        self.config = config
        self.log = logging.getLogger(LOG_CATEGORY)
        self.smpp = None
        self.bindDeferred = None

    def connect(self):
        factory = SMPPClientFactory(self.config)
        if self.config.useSSL:
            self.log.warning('Establishing SSL connection to %s:%d' % (self.config.host, self.config.port))
            #pylint: disable=no-member
            reactor.connectSSL(self.config.host, self.config.port, factory, CtxFactory(self.config))
        else:
            self.log.warning('Establishing TCP connection to %s:%d' % (self.config.host, self.config.port))
            #pylint: disable=no-member
            reactor.connectTCP(self.config.host, self.config.port, factory)
        return factory.buildProtocolDeferred.addCallback(self.onConnect)

    def onConnect(self, smpp):
        self.smpp = smpp
        if self.msgHandler is not None:
            smpp.setDataRequestHandler(self.msgHandler)
        return smpp

    def connectAndBind(self):
        self.bindDeferred = defer.Deferred()
        self.connect().addCallback(self.doBind).addErrback(self.bindDeferred.errback)
        return self.bindDeferred

    def doBind(self, smpp):
        self.bind(smpp).addCallback(self.bound).addErrback(self.bindFailed, smpp)
        return smpp

    def bind(self, smpp):
        raise NotImplementedError()

    #If bind fails, don't errback until we're disconnected
    def bindFailed(self, error, smpp):
        smpp.getDisconnectedDeferred().addCallback(lambda result: self.bindDeferred.errback(error))

    def bound(self, result):
        self.bindDeferred.callback(result.smpp)

class SMPPClientTransmitter(SMPPClientBase):

    def bind(self, smpp):
        return smpp.bindAsTransmitter()

class SMPPClientReceiver(SMPPClientBase):

    def __init__(self, config, msgHandler):
        SMPPClientBase.__init__(self, config)
        self.msgHandler = msgHandler

    def bind(self, smpp):
        return smpp.bindAsReceiver(self.msgHandler)

class SMPPClientTransceiver(SMPPClientReceiver):

    def bind(self, smpp):
        return smpp.bindAsTransceiver(self.msgHandler)

#TODO - move this to mozes code base since
# the service support in Twisted is so crappy
class SMPPClientService(Service):

    def __init__(self, smppClient):
        self.client = smppClient
        self.stopDeferred = defer.Deferred()
        self.log = logging.getLogger(LOG_CATEGORY)

    def getStopDeferred(self):
        return self.stopDeferred

    @defer.inlineCallbacks
    def startService(self):
        Service.startService(self)
        bindDeferred = self.client.connectAndBind()
        bindDeferred.addErrback(self.handleStartError)
        smpp = yield bindDeferred
        smpp.getDisconnectedDeferred().chainDeferred(self.stopDeferred)
        defer.returnValue(smpp)

    def handleStartError(self, error):
        self.stopDeferred.errback(error)
        return error

    def stopService(self):
        Service.stopService(self)
        if self.client.smpp:
            self.log.info("Stopping SMPP Client")
            return self.client.smpp.unbindAndDisconnect()




import os
from setuptools import setup, find_packages


def parse_requirements(filename):
    """load requirements from a pip requirements file"""
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and (not line.startswith("#") and not line.startswith('-'))]


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="smpp.twisted3",
    version="0.8",
    author="Roger Hoover",
    author_email="roger.hoover@gmail.com",
    description="SMPP 3.4 client built on Twisted / Python3",
    license='Apache License 2.0',
    packages=find_packages(exclude=["tests"]),
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    keywords="smpp twisted",
    url="https://github.com/jookies/smpp.twisted",
    py_modules=["smpp.twisted3"],
    include_package_data=True,
    package_data={'smpp.twisted3': ['README.md']},
    zip_safe=False,
    install_requires=parse_requirements('requirements.txt'),
    tests_require=[
        'mock',
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Twisted",
        "Topic :: System :: Networking",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



"""
Copyright 2009-2010 Mozes, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expressed or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import logging
import binascii
from twisted.trial.unittest import TestCase
from twisted.internet import error, defer
from mock import Mock, sentinel
import sys

sys.path.append(".")
sys.path.append("..")
sys.path.append("../..")
from smpp.pdu.error import (
    SMPPClientConnectionCorruptedError,
    SMPPClientSessionStateError,
    SMPPClientError
)
from smpp.pdu.operations import (
    SubmitSM,
    GenericNack,
    DeliverSM,
    DeliverSMResp,
    Unbind,
    UnbindResp,
    DataSM,
    DataSMResp
)
from smpp.pdu.pdu_types import CommandStatus, AddrTon, AddrNpi, DeliveryFailureReason
from smpp.twisted.config import SMPPClientConfig
from smpp.twisted.protocol import SMPPClientProtocol, SMPPSessionStates, SMPPOutboundTxnResult, DataHandlerResponse

class FakeClientError(SMPPClientError):
    pass

class ProtocolTestCase(TestCase):

    def getProtocolObject(self):
        smpp = SMPPClientProtocol()
        config = SMPPClientConfig(
            host='localhost',
            port = 82,
            username = '',
            password = '',
        )
        smpp.config = Mock(return_value=config)
        return smpp

    def test_corruptData(self):
        smpp = self.getProtocolObject()
        self.assertEquals(b'', smpp.recvBuffer)
        smpp.sendPDU = Mock()
        smpp.cancelOutboundTransactions = Mock()
        smpp.shutdown = Mock()
        smpp.sessionState = SMPPSessionStates.BOUND_TRX
        smpp.corruptDataRecvd()
        #Assert that corrupt data
        #Triggers a shutdown call
        self.assertEquals(1, smpp.shutdown.call_count)
        #Causes outbound transactions to be canceled
        self.assertEquals(1, smpp.cancelOutboundTransactions.call_count)
        #Responds with a generic nack with invalid cmd len error status
        nackResp = smpp.sendPDU.call_args[0][0]
        self.assertEquals(GenericNack(seqNum=None, status=CommandStatus.ESME_RINVCMDLEN), nackResp)
        #Causes new data received to be ignored
        newDataHex = b'afc4'
        smpp.dataReceived(binascii.a2b_hex(newDataHex))
        self.assertEquals(newDataHex, binascii.b2a_hex(smpp.recvBuffer))
        #Causes new data requests to fail immediately
        submitPdu = SubmitSM(
            source_addr_ton=AddrTon.ALPHANUMERIC,
            source_addr='mobileway',
            dest_addr_ton=AddrTon.INTERNATIONAL,
            dest_addr_npi=AddrNpi.ISDN,
            destination_addr='1208230',
            short_message='HELLO',
        )
        return self.assertFailure(smpp.sendDataRequest(submitPdu), SMPPClientConnectionCorruptedError)

    def test_cancelOutboundTransactions(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        smpp.sessionState = SMPPSessionStates.BOUND_TRX
        #start two transactions
        submitPdu1 = SubmitSM(
            source_addr_ton=AddrTon.ALPHANUMERIC,
            source_addr='mobileway',
            dest_addr_ton=AddrTon.INTERNATIONAL,
            dest_addr_npi=AddrNpi.ISDN,
            destination_addr='1208230',
            short_message='HELLO1',
        )
        submitPdu2 = SubmitSM(
            source_addr_ton=AddrTon.ALPHANUMERIC,
            source_addr='mobileway',
            dest_addr_ton=AddrTon.INTERNATIONAL,
            dest_addr_npi=AddrNpi.ISDN,
            destination_addr='1208230',
            short_message='HELLO2',
        )
        d1 = smpp.sendDataRequest(submitPdu1)
        d2 = smpp.sendDataRequest(submitPdu2)
        self.assertEquals(2, len(smpp.outTxns))
        smpp.cancelOutboundTransactions(FakeClientError('test'))
        return defer.DeferredList([
            self.assertFailure(d1, FakeClientError),
            self.assertFailure(d2, FakeClientError),
        ])

    def test_finish_txns(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        smpp.sessionState = SMPPSessionStates.BOUND_TRX

        #setup outbound txns
        outPdu1 = SubmitSM(
            seqNum=98790,
            source_addr='mobileway',
            destination_addr='1208230',
            short_message='HELLO1',
        )
        outRespPdu1 = outPdu1.requireAck(seqNum=outPdu1.seqNum)

        outPdu2 = SubmitSM(
            seqNum=875,
            source_addr='mobileway',
            destination_addr='1208230',
            short_message='HELLO1',
        )
        outRespPdu2 = outPdu2.requireAck(seqNum=outPdu2.seqNum, status=CommandStatus.ESME_RINVSRCTON)

        outDeferred1 = smpp.startOutboundTransaction(outPdu1, 1)
        outDeferred2 = smpp.startOutboundTransaction(outPdu2, 1)

        finishOutTxns = smpp.finishOutboundTxns()

        #Simulate second txn having error
        smpp.endOutboundTransactionErr(outRespPdu2, FakeClientError('test'))
        #Assert txns not done yet
        self.assertFalse(finishOutTxns.called)

        #Simulate first txn finishing
        smpp.endOutboundTransaction(outRespPdu1)
        #Assert txns are all done
        self.assertTrue(finishOutTxns.called)

        return defer.DeferredList([
            outDeferred1,
            self.assertFailure(outDeferred2, FakeClientError),
            finishOutTxns,
        ]
        )

    def test_graceful_unbind(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        smpp.sessionState = SMPPSessionStates.BOUND_TRX

        #setup outbound txn
        outPdu = SubmitSM(
            seqNum=98790,
            source_addr='mobileway',
            destination_addr='1208230',
            short_message='HELLO1',
        )
        outRespPdu = outPdu.requireAck(seqNum=outPdu.seqNum)
        outDeferred = smpp.startOutboundTransaction(outPdu, 1)
        #setup inbound txn
        inPdu = DeliverSM(
            seqNum=764765,
            source_addr='mobileway',
            destination_addr='1208230',
            short_message='HELLO1',
        )
        inDeferred = smpp.startInboundTransaction(inPdu)

        #Call unbind
        unbindDeferred = smpp.unbind()

        #Assert unbind request not sent and deferred not fired
        self.assertEquals(0, smpp.sendPDU.call_count)
        self.assertFalse(unbindDeferred.called)

        #Simulate inbound txn finishing
        smpp.endInboundTransaction(inPdu)

        #Assert unbind request not sent and deferred not fired
        self.assertEquals(0, smpp.sendPDU.call_count)
        self.assertFalse(unbindDeferred.called)

        #Simulate outbound txn finishing
        smpp.endOutboundTransaction(outRespPdu)

        #Assert unbind request was sent but deferred not yet fired
        self.assertEquals(1, smpp.sendPDU.call_count)
        sentPdu = smpp.sendPDU.call_args[0][0]
        self.assertTrue(isinstance(sentPdu, Unbind))
        self.assertFalse(unbindDeferred.called)

        bindResp = UnbindResp(seqNum=sentPdu.seqNum)

        #Simulate unbind_resp
        smpp.endOutboundTransaction(bindResp)

        #Assert unbind deferred fired
        self.assertTrue(unbindDeferred.called)
        self.assertTrue(isinstance(unbindDeferred.result, SMPPOutboundTxnResult))
        expectedResult = SMPPOutboundTxnResult(smpp, sentPdu, bindResp)
        self.assertEquals(expectedResult, unbindDeferred.result)

    def test_bind_when_not_in_open_state(self):
        smpp = self.getProtocolObject()
        smpp.sessionState = SMPPSessionStates.BOUND_TRX
        return self.assertFailure(smpp.bindAsTransmitter(), SMPPClientSessionStateError)

    def test_unbind_when_not_bound(self):
        smpp = self.getProtocolObject()
        smpp.sessionState = SMPPSessionStates.BIND_TX_PENDING
        return self.assertFailure(smpp.unbind(), SMPPClientSessionStateError)

    def test_server_initiated_unbind_cancels_enquire_link_timer(self):
        smpp = self.getProtocolObject()
        smpp.sendResponse = Mock()
        smpp.disconnect = Mock()

        smpp.sessionState = SMPPSessionStates.BOUND_TRX
        smpp.activateEnquireLinkTimer()
        self.assertNotEquals(None, smpp.enquireLinkTimer)
        smpp.onPDURequest_unbind(Unbind())
        self.assertEquals(None, smpp.enquireLinkTimer)

    def test_sendDataRequest_when_not_bound(self):
        smpp = self.getProtocolObject()
        smpp.sessionState = SMPPSessionStates.BIND_TX_PENDING
        return self.assertFailure(smpp.sendDataRequest(SubmitSM()), SMPPClientSessionStateError)

    def test_sendDataRequest_invalid_pdu(self):
        smpp = self.getProtocolObject()
        smpp.sessionState = SMPPSessionStates.BOUND_TRX
        return self.assertFailure(smpp.sendDataRequest(Unbind()), SMPPClientError)

    def test_data_handler_return_none(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        reqPDU = DeliverSM(5)
        smpp.PDURequestSucceeded(None, reqPDU)
        self.assertEquals(1, smpp.sendPDU.call_count)
        sent = smpp.sendPDU.call_args[0][0]
        self.assertEquals(DeliverSMResp(5), sent)

    def test_data_handler_return_status(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        reqPDU = DeliverSM(5)
        smpp.PDURequestSucceeded(CommandStatus.ESME_RINVSRCTON, reqPDU)
        self.assertEquals(1, smpp.sendPDU.call_count)
        sent = smpp.sendPDU.call_args[0][0]
        self.assertEquals(DeliverSMResp(5, CommandStatus.ESME_RINVSRCTON), sent)

    def test_data_handler_return_resp(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        reqPDU = DataSM(6)
        smpp.PDURequestSucceeded(DataHandlerResponse(CommandStatus.ESME_RINVSRCTON, delivery_failure_reason=DeliveryFailureReason.PERMANENT_NETWORK_ERROR), reqPDU)
        self.assertEquals(1, smpp.sendPDU.call_count)
        sent = smpp.sendPDU.call_args[0][0]
        self.assertEquals(DataSMResp(6, CommandStatus.ESME_RINVSRCTON, delivery_failure_reason=DeliveryFailureReason.PERMANENT_NETWORK_ERROR), sent)

    def test_data_handler_return_junk(self):
        smpp = self.getProtocolObject()
        smpp.sendPDU = Mock()
        smpp.shutdown = Mock()
        reqPDU = DeliverSM(5)
        smpp.PDURequestSucceeded(3, reqPDU)
        self.assertEquals(1, smpp.shutdown.call_count)
        self.assertEquals(1, smpp.sendPDU.call_count)
        sent = smpp.sendPDU.call_args[0][0]
        self.assertEquals(DeliverSMResp(5, CommandStatus.ESME_RX_T_APPN), sent)


"""
Copyright 2009-2010 Mozes, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either expressed or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import logging
import functools
from twisted.trial.unittest import TestCase
from twisted.internet import error, reactor, defer
from twisted.internet.protocol import Factory
import mock
import sys

sys.path.append(".")
sys.path.append("..")
sys.path.append("../..")
import tests.smsc_simulator as smsc_simulator
from smpp.twisted.config import SMPPClientConfig

from smpp.twisted.protocol import SMPPClientProtocol
from smpp.twisted.client import SMPPClientTransmitter, SMPPClientReceiver, SMPPClientTransceiver, DataHandlerResponse, SMPPClientService

from smpp.pdu.error import (
    SMPPClientConnectionCorruptedError,
    PDUCorruptError,
    PDUParseError,
    SMPPClientSessionStateError,
    SessionStateError,
    SMPPProtocolError,
    SMPPClientError,
    SMPPError,
    SMPPGenericNackTransactionError,
    SMPPTransactionError,
    SMPPRequestTimoutError,
    SMPPSessionInitTimoutError
)
from smpp.pdu.operations import (
    GenericNack,
    getPDUClass,
    EnquireLink,
    BindTransmitter,
    BindTransceiver,
    BindReceiver,
    Unbind,
    SubmitSM,
    UnbindResp,
    EnquireLinkResp,
    DataSM,
    AlertNotification,
    DeliverSM,
    QuerySMResp,
    DataSMResp
)
from smpp.pdu.pdu_types import DeliveryFailureReason, CommandStatus

class SimulatorTestCase(TestCase):
    protocol = smsc_simulator.BlackHoleSMSC
    configArgs = {}

    def setUp(self):
        self.factory = Factory()
        self.factory.protocol = self.protocol
        self.port = reactor.listenTCP(0, self.factory)
        self.testPort = self.port.getHost().port

        args = self.configArgs.copy()
        args['host'] = self.configArgs.get('host', 'localhost')
        args['port'] = self.configArgs.get('port', self.testPort)
        args['username'] = self.configArgs.get('username', '')
        args['password'] = self.configArgs.get('password', '')

        self.config = SMPPClientConfig(**args)

    def tearDown(self):
        self.port.stopListening()

class SessionInitTimeoutTestCase(SimulatorTestCase):
    configArgs = {
        'sessionInitTimerSecs': 0.1,
    }

    def test_bind_transmitter_timeout(self):
        client = SMPPClientTransmitter(self.config)
        return self.assertFailure(client.connectAndBind(), SMPPSessionInitTimoutError)

    def test_bind_receiver_timeout(self):
        client = SMPPClientReceiver(self.config, lambda smpp, pdu: None)
        return self.assertFailure(client.connectAndBind(), SMPPSessionInitTimoutError)

    def test_bind_transceiver_timeout(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        return self.assertFailure(client.connectAndBind(), SMPPSessionInitTimoutError)

class BindErrorTestCase(SimulatorTestCase):
    protocol = smsc_simulator.BindErrorSMSC

    def test_bind_error(self):
        client = SMPPClientTransmitter(self.config)
        return self.assertFailure(client.connectAndBind(), SMPPTransactionError)

class BindErrorGenericNackTestCase(SimulatorTestCase):
    protocol = smsc_simulator.BindErrorGenericNackSMSC

    def test_bind_error_generic_nack(self):
        client = SMPPClientTransmitter(self.config)
        return self.assertFailure(client.connectAndBind(), SMPPGenericNackTransactionError)

class UnbindTimeoutTestCase(SimulatorTestCase):
    protocol = smsc_simulator.UnbindNoResponseSMSC
    configArgs = {
        'sessionInitTimerSecs': 0.1,
    }

    def setUp(self):
        SimulatorTestCase.setUp(self)
        self.unbindDeferred = defer.Deferred()

    def test_unbind_timeout(self):
        client = SMPPClientTransmitter(self.config)
        bindDeferred = client.connectAndBind().addCallback(self.do_unbind)
        return defer.DeferredList([
            bindDeferred, #asserts that bind was successful
            self.assertFailure(self.unbindDeferred, SMPPSessionInitTimoutError), #asserts that unbind timed out
            ]
        )

    def do_unbind(self, smpp):
        smpp.unbind().chainDeferred(self.unbindDeferred)
        return smpp

class ResponseTimeoutTestCase(SimulatorTestCase):
    protocol = smsc_simulator.NoResponseOnSubmitSMSC
    configArgs = {
        'responseTimerSecs': 0.1,
    }

    def setUp(self):
        SimulatorTestCase.setUp(self)
        self.disconnectDeferred = defer.Deferred()
        self.submitSMDeferred1 = defer.Deferred()
        self.submitSMDeferred2 = defer.Deferred()

    def test_response_timeout(self):
        client = SMPPClientTransmitter(self.config)
        bindDeferred = client.connectAndBind().addCallback(self.do_test_setup)
        self.disconnectDeferred.addCallback(self.verify)
        return defer.DeferredList([
            bindDeferred, #asserts that bind was successful
            self.assertFailure(self.submitSMDeferred1, SMPPRequestTimoutError), #asserts that request1 timed out
            self.assertFailure(self.submitSMDeferred2, SMPPRequestTimoutError), #asserts that request2 timed out
            self.disconnectDeferred, #asserts that disconnect deferred was triggered
            ]
        )

    def do_test_setup(self, smpp):
        self.smpp = smpp
        smpp.getDisconnectedDeferred().chainDeferred(self.disconnectDeferred)
        smpp.sendDataRequest(SubmitSM(source_addr='t1', destination_addr='1208230', short_message='HELLO')).chainDeferred(self.submitSMDeferred1)
        smpp.sendDataRequest(SubmitSM(source_addr='t2', destination_addr='1208230', short_message='HELLO')).chainDeferred(self.submitSMDeferred2)
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)
        return smpp

    #Test unbind sent
    def verify(self, result):
        self.assertEquals(1, self.smpp.sendPDU.call_count)
        sent = self.smpp.sendPDU.call_args[0][0]
        self.assertTrue(isinstance(sent, Unbind))

class InactivityTimeoutTestCase(SimulatorTestCase):
    protocol = smsc_simulator.HappySMSC
    configArgs = {
        'inactivityTimerSecs': 0.1,
    }

    def setUp(self):
        SimulatorTestCase.setUp(self)
        self.disconnectDeferred = defer.Deferred()

    def test_inactivity_timeout(self):
        client = SMPPClientTransmitter(self.config)
        bindDeferred = client.connectAndBind().addCallback(self.do_test_setup)
        self.disconnectDeferred.addCallback(self.verify)
        return defer.DeferredList([
            bindDeferred, #asserts that bind was successful
            self.disconnectDeferred, #asserts that disconnect deferred was triggered
            ]
        )

    def do_test_setup(self, smpp):
        self.smpp = smpp
        smpp.getDisconnectedDeferred().chainDeferred(self.disconnectDeferred)
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)
        return smpp

    #Test unbind sent
    def verify(self, result):
        self.assertEquals(1, self.smpp.sendPDU.call_count)
        sent = self.smpp.sendPDU.call_args[0][0]
        self.assertTrue(isinstance(sent, Unbind))

class ServerInitiatedUnbindTestCase(SimulatorTestCase):
    protocol = smsc_simulator.UnbindOnSubmitSMSC

    def setUp(self):
        SimulatorTestCase.setUp(self)
        self.disconnectDeferred = defer.Deferred()
        self.submitSMDeferred = defer.Deferred()

    def test_server_unbind(self):
        client = SMPPClientTransmitter(self.config)
        bindDeferred = client.connectAndBind().addCallback(self.mock_stuff)
        self.disconnectDeferred.addCallback(self.verify)
        return defer.DeferredList([
            bindDeferred, #asserts that bind was successful
            self.disconnectDeferred, #asserts that disconnect deferred was triggered,
            self.assertFailure(self.submitSMDeferred, SMPPClientSessionStateError), #asserts that outbound txn was canceled
            ]
        )

    def mock_stuff(self, smpp):
        self.smpp = smpp
        smpp.getDisconnectedDeferred().chainDeferred(self.disconnectDeferred)
        smpp.sendDataRequest(SubmitSM(source_addr='mobileway', destination_addr='1208230', short_message='HELLO1')).chainDeferred(self.submitSMDeferred)
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)
        return smpp

    def verify(self, result):
        self.assertEquals(1, self.smpp.sendPDU.call_count)
        sent = self.smpp.sendPDU.call_args[0][0]
        self.assertEquals(UnbindResp(1), sent)

class EnquireLinkTestCase(SimulatorTestCase):
    protocol = smsc_simulator.EnquireLinkEchoSMSC
    configArgs = {
        'enquireLinkTimerSecs': 0.1,
    }

    @defer.inlineCallbacks
    def test_enquire_link(self):
        client = SMPPClientTransmitter(self.config)
        smpp = yield client.connect()
        #Assert that enquireLinkTimer is not yet active on connection
        self.assertEquals(None, smpp.enquireLinkTimer)

        bindDeferred = client.bind(smpp)
        #Assert that enquireLinkTimer is not yet active until bind is complete
        self.assertEquals(None, smpp.enquireLinkTimer)
        yield bindDeferred
        #Assert that enquireLinkTimer is now active after bind is complete
        self.assertNotEquals(None, smpp.enquireLinkTimer)

        #Wrap functions for tracking
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)
        smpp.PDUReceived = mock.Mock(wraps=smpp.PDUReceived)

        yield self.wait(0.25)

        self.verifyEnquireLink(smpp)

        #Assert that enquireLinkTimer is still active
        self.assertNotEquals(None, smpp.enquireLinkTimer)

        unbindDeferred = smpp.unbind()

        #Assert that enquireLinkTimer is no longer active after unbind is issued
        self.assertEquals(None, smpp.enquireLinkTimer)

        yield unbindDeferred
        #Assert that enquireLinkTimer is no longer active after unbind is complete
        self.assertEquals(None, smpp.enquireLinkTimer)
        yield smpp.disconnect()

    def wait(self, time_secs):
        finished = defer.Deferred()
        reactor.callLater(time_secs, finished.callback, None)
        return finished

    def verifyEnquireLink(self, smpp):
        self.assertEquals(4, smpp.sendPDU.call_count)
        self.assertEquals(4, smpp.PDUReceived.call_count)
        sent1 = smpp.sendPDU.call_args_list[0][0][0]
        sent2 = smpp.sendPDU.call_args_list[1][0][0]
        sent3 = smpp.sendPDU.call_args_list[2][0][0]
        sent4 = smpp.sendPDU.call_args_list[3][0][0]
        recv1 = smpp.PDUReceived.call_args_list[0][0][0]
        recv2 = smpp.PDUReceived.call_args_list[1][0][0]
        recv3 = smpp.PDUReceived.call_args_list[2][0][0]
        recv4 = smpp.PDUReceived.call_args_list[3][0][0]

        self.assertEquals(EnquireLink(2), sent1)
        self.assertEquals(EnquireLinkResp(2), recv1)

        self.assertEquals(EnquireLink(1), recv2)
        self.assertEquals(EnquireLinkResp(1), sent2)

        self.assertEquals(EnquireLink(3), sent3)
        self.assertEquals(EnquireLinkResp(3), recv3)

        self.assertEquals(EnquireLink(2), recv4)
        self.assertEquals(EnquireLinkResp(2), sent4)

class TransmitterLifecycleTestCase(SimulatorTestCase):
    protocol = smsc_simulator.HappySMSC

    def setUp(self):
        SimulatorTestCase.setUp(self)
        self.unbindDeferred = defer.Deferred()
        self.submitSMDeferred = defer.Deferred()
        self.disconnectDeferred = defer.Deferred()

    def test_unbind(self):
        client = SMPPClientTransmitter(self.config)
        bindDeferred = client.connectAndBind().addCallback(self.do_lifecycle)
        return defer.DeferredList([
            bindDeferred, #asserts that bind was successful
            self.submitSMDeferred, #asserts that submit was successful
            self.unbindDeferred, #asserts that unbind was successful
            self.disconnectDeferred, #asserts that disconnect was successful
            ]
        )

    def do_lifecycle(self, smpp):
        smpp.getDisconnectedDeferred().chainDeferred(self.disconnectDeferred)
        smpp.sendDataRequest(SubmitSM()).chainDeferred(self.submitSMDeferred).addCallback(lambda result: smpp.unbindAndDisconnect().chainDeferred(self.unbindDeferred))
        return smpp

class AlertNotificationTestCase(SimulatorTestCase):
    protocol = smsc_simulator.AlertNotificationSMSC

    @defer.inlineCallbacks
    def test_alert_notification(self):
        client = SMPPClientTransmitter(self.config)
        smpp = yield client.connectAndBind()

        alertNotificationDeferred = defer.Deferred()
        alertHandler = mock.Mock(wraps=lambda smpp, pdu: alertNotificationDeferred.callback(None))
        smpp.setAlertNotificationHandler(alertHandler)

        sendDataDeferred = smpp.sendDataRequest(DataSM())

        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)
        smpp.PDUReceived = mock.Mock(wraps=smpp.PDUReceived)

        yield sendDataDeferred
        yield alertNotificationDeferred
        yield smpp.unbindAndDisconnect()

        self.assertEquals(1, alertHandler.call_count)
        self.assertEquals(smpp, alertHandler.call_args[0][0])
        self.assertTrue(isinstance(alertHandler.call_args[0][1], AlertNotification))

        self.assertEquals(1, smpp.sendPDU.call_count)
        self.assertEquals(3, smpp.PDUReceived.call_count)
        sent1 = smpp.sendPDU.call_args[0][0]
        recv1 = smpp.PDUReceived.call_args_list[0][0][0]
        recv2 = smpp.PDUReceived.call_args_list[1][0][0]
        recv3 = smpp.PDUReceived.call_args_list[2][0][0]
        self.assertTrue(isinstance(recv1, DataSMResp))
        self.assertTrue(isinstance(recv2, AlertNotification))
        self.assertTrue(isinstance(sent1, Unbind))
        self.assertTrue(isinstance(recv3, UnbindResp))

class CommandLengthTooShortTestCase(SimulatorTestCase):
    protocol = smsc_simulator.CommandLengthTooShortSMSC

    @defer.inlineCallbacks
    def test_generic_nack_on_invalid_cmd_len(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        msgSentDeferred = defer.Deferred()

        smpp.sendPDU = mock.Mock()
        smpp.sendPDU.side_effect = functools.partial(self.mock_side_effect, msgSentDeferred)

        yield msgSentDeferred

        yield smpp.disconnect()

        self.assertEquals(1, smpp.sendPDU.call_count)
        smpp.sendPDU.assert_called_with(GenericNack(status=CommandStatus.ESME_RINVMSGLEN))

    def mock_side_effect(self, msgSentDeferred, pdu):
        msgSentDeferred.callback(None)
        return mock.DEFAULT

class CommandLengthTooLongTestCase(SimulatorTestCase):
    protocol = smsc_simulator.CommandLengthTooLongSMSC
    configArgs = {
        'pduReadTimerSecs': 0.1,
    }

    @defer.inlineCallbacks
    def test_generic_nack_on_invalid_cmd_len(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        msgSentDeferred = defer.Deferred()

        smpp.sendPDU = mock.Mock()
        smpp.sendPDU.side_effect = functools.partial(self.mock_side_effect, msgSentDeferred)

        yield msgSentDeferred

        yield smpp.disconnect()

        self.assertEquals(1, smpp.sendPDU.call_count)
        smpp.sendPDU.assert_called_with(GenericNack(status=CommandStatus.ESME_RINVCMDLEN))

    def mock_side_effect(self, msgSentDeferred, pdu):
        msgSentDeferred.callback(None)
        return mock.DEFAULT

class InvalidCommandIdTestCase(SimulatorTestCase):
    protocol = smsc_simulator.InvalidCommandIdSMSC

    @defer.inlineCallbacks
    def test_generic_nack_on_invalid_cmd_id(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        msgSentDeferred = defer.Deferred()

        smpp.sendPDU = mock.Mock()
        smpp.sendPDU.side_effect = functools.partial(self.mock_side_effect, msgSentDeferred)

        yield msgSentDeferred

        yield smpp.disconnect()

        self.assertEquals(1, smpp.sendPDU.call_count)
        smpp.sendPDU.assert_called_with(GenericNack(status=CommandStatus.ESME_RINVCMDID))

    def mock_side_effect(self, msgSentDeferred, pdu):
        msgSentDeferred.callback(None)
        return mock.DEFAULT

class NonFatalParseErrorTestCase(SimulatorTestCase):
    protocol = smsc_simulator.NonFatalParseErrorSMSC

    @defer.inlineCallbacks
    def test_nack_on_invalid_msg(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        msgSentDeferred = defer.Deferred()

        smpp.sendPDU = mock.Mock()
        smpp.sendPDU.side_effect = functools.partial(self.mock_side_effect, msgSentDeferred)

        yield msgSentDeferred

        yield smpp.disconnect()

        self.assertEquals(1, smpp.sendPDU.call_count)
        smpp.sendPDU.assert_called_with(QuerySMResp(seqNum=self.protocol.seqNum, status=CommandStatus.ESME_RINVSRCTON))

    def mock_side_effect(self, msgSentDeferred, pdu):
        msgSentDeferred.callback(None)
        return mock.DEFAULT

class GenericNackNoSeqNumTestCase(SimulatorTestCase):
    protocol = smsc_simulator.GenericNackNoSeqNumOnSubmitSMSC

    @defer.inlineCallbacks
    def test_generic_nack_no_seq_num(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        try:
            submitDeferred = smpp.sendDataRequest(SubmitSM())
            smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)
            yield submitDeferred
        except SMPPClientConnectionCorruptedError:
            pass
        else:
            self.assertTrue(False, "SMPPClientConnectionCorruptedError not raised")

        #for nack with no seq num, the connection is corrupt so don't unbind()
        self.assertEquals(0, smpp.sendPDU.call_count)

class GenericNackWithSeqNumTestCase(SimulatorTestCase):
    protocol = smsc_simulator.GenericNackWithSeqNumOnSubmitSMSC

    @defer.inlineCallbacks
    def test_generic_nack_no_seq_num(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        try:
            yield smpp.sendDataRequest(SubmitSM())
        except SMPPGenericNackTransactionError:
            pass
        else:
            self.assertTrue(False, "SMPPGenericNackTransactionError not raised")
        finally:
            yield smpp.unbindAndDisconnect()

class ErrorOnSubmitTestCase(SimulatorTestCase):
    protocol = smsc_simulator.ErrorOnSubmitSMSC

    @defer.inlineCallbacks
    def test_error_on_submit(self):
        client = SMPPClientTransceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()
        try:
            yield smpp.sendDataRequest(SubmitSM())
        except SMPPTransactionError:
            pass
        else:
            self.assertTrue(False, "SMPPTransactionError not raised")
        finally:
            yield smpp.unbindAndDisconnect()

class ReceiverLifecycleTestCase(SimulatorTestCase):
    protocol = smsc_simulator.DeliverSMAndUnbindSMSC

    @defer.inlineCallbacks
    def test_receiver_lifecycle(self):
        client = SMPPClientReceiver(self.config, lambda smpp, pdu: None)
        smpp = yield client.connectAndBind()

        smpp.PDUReceived = mock.Mock(wraps=smpp.PDUReceived)
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)

        yield smpp.getDisconnectedDeferred()

        self.assertEquals(2, smpp.PDUReceived.call_count)
        self.assertEquals(2, smpp.sendPDU.call_count)
        recv1 = smpp.PDUReceived.call_args_list[0][0][0]
        recv2 = smpp.PDUReceived.call_args_list[1][0][0]
        sent1 = smpp.sendPDU.call_args_list[0][0][0]
        sent2 = smpp.sendPDU.call_args_list[1][0][0]
        self.assertTrue(isinstance(recv1, DeliverSM))
        self.assertEquals(recv1.requireAck(recv1.seqNum), sent1)
        self.assertTrue(isinstance(recv2, Unbind))
        self.assertEquals(recv2.requireAck(recv2.seqNum), sent2)

class ReceiverDataHandlerExceptionTestCase(SimulatorTestCase):
    protocol = smsc_simulator.DeliverSMSMSC

    @defer.inlineCallbacks
    def test_receiver_exception(self):
        client = SMPPClientReceiver(self.config, self.barf)
        smpp = yield client.connectAndBind()

        smpp.PDUReceived = mock.Mock(wraps=smpp.PDUReceived)
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)

        yield smpp.getDisconnectedDeferred()

        self.assertEquals(2, smpp.PDUReceived.call_count)
        self.assertEquals(2, smpp.sendPDU.call_count)
        recv1 = smpp.PDUReceived.call_args_list[0][0][0]
        recv2 = smpp.PDUReceived.call_args_list[1][0][0]
        sent1 = smpp.sendPDU.call_args_list[0][0][0]
        sent2 = smpp.sendPDU.call_args_list[1][0][0]
        self.assertTrue(isinstance(recv1, DeliverSM))
        self.assertEquals(recv1.requireAck(recv1.seqNum, CommandStatus.ESME_RX_T_APPN), sent1)
        self.assertTrue(isinstance(sent2, Unbind))
        self.assertTrue(isinstance(recv2, UnbindResp))

    def barf(self, smpp, pdu):
        raise ValueError('barf')

class ReceiverDataHandlerBadResponseParamTestCase(SimulatorTestCase):
    protocol = smsc_simulator.DeliverSMSMSC

    @defer.inlineCallbacks
    def test_receiver_bad_resp_param(self):
        client = SMPPClientReceiver(self.config, self.respondBadParam)
        smpp = yield client.connectAndBind()

        smpp.PDUReceived = mock.Mock(wraps=smpp.PDUReceived)
        smpp.sendPDU = mock.Mock(wraps=smpp.sendPDU)

        yield smpp.getDisconnectedDeferred()

        self.assertEquals(2, smpp.PDUReceived.call_count)
        self.assertEquals(2, smpp.sendPDU.call_count)
        recv1 = smpp.PDUReceived.call_args_list[0][0][0]
        recv2 = smpp.PDUReceived.call_args_list[1][0][0]
        sent1 = smpp.sendPDU.call_args_list[0][0][0]
        sent2 = smpp.sendPDU.call_args_list[1][0][0]
        self.assertTrue(isinstance(recv1, DeliverSM))
        self.assertEquals(recv1.requireAck(recv1.seqNum, CommandStatus.ESME_RX_T_APPN), sent1)
        self.assertTrue(isinstance(sent2, Unbind))
        self.assertTrue(isinstance(recv2, UnbindResp))

    def respondBadParam(self, smpp, pdu):
        return DataHandlerResponse(delivery_failure_reason=DeliveryFailureReason.PERMANENT_NETWORK_ERROR)

class ReceiverUnboundErrorTestCase(SimulatorTestCase):
    protocol = smsc_simulator.DeliverSMBeforeBoundSMSC

    def setUp(self):
        SimulatorTestCase.setUp(self)

    def test_receiver_exception(self):
        client = SMPPClientReceiver(self.config, lambda smpp, pdu: None)
        bindDeferred = client.connectAndBind()
        return self.assertFailure(bindDeferred, SessionStateError)

class OutbindTestCase(SimulatorTestCase):
    protocol = smsc_simulator.OutbindSMSC

    def msgHandler(self, smpp, pdu):
        smpp.unbindAndDisconnect()
        return None

    @defer.inlineCallbacks
    def test_outbind(self):
        client = SMPPClientReceiver(self.config, self.msgHandler)
        smpp = yield client.connect()
        yield smpp.getDisconnectedDeferred()

class SMPPClientServiceBindTimeoutTestCase(SimulatorTestCase):
    configArgs = {
        'sessionInitTimerSecs': 0.1,
    }

    def test_bind_transmitter_timeout(self):
        client = SMPPClientTransmitter(self.config)
        svc = SMPPClientService(client)
        stopDeferred = svc.getStopDeferred()
        startDeferred = svc.startService()
        return defer.DeferredList([
            self.assertFailure(startDeferred, SMPPSessionInitTimoutError),
            self.assertFailure(stopDeferred, SMPPSessionInitTimoutError),
        ])